package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.engine.particle.PurplePowderExplosion;
import com.pancik.libgdx.zumpatalentov.engine.particle.SmallExplosion;
import com.pancik.libgdx.zumpatalentov.engine.player.PlayablePlayer;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager.CostWrapper;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconButtonClickable;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconSelection;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class BaseBuildingSpot extends BuildingSpotEntity {
    public static enum PossibleBaseBuildings {
        FRANKIE_FACTORY (new CostWrapper(40, 0)),
        MUTATE_LAB (new CostWrapper(80, 0));
        
        private CostWrapper cost;

        private PossibleBaseBuildings(CostWrapper cost) {
            this.cost = cost;
        }

        public CostWrapper getCost() {
            return cost;
        }

        public PossibilityCarrier getPossibilityCarrier() {
            PossibilityCarrier out = new PossibilityCarrier();
            out.name = name();
            out.cost = cost;
            return out;
        }
    }
    
    public static Texture BASE_BUILDING_SPOT_TEXTURE = DrawableData.textures.get("base_building_spot");
    
    public static final int TEXTURE_SIZE_X = 128;
    public static final int TEXTURE_SIZE_Y = 128;
    public static final float DEFAULT_SIZE_X = 128.0f;
    public static final float DEFAULT_SIZE_Y = 128.0f;
    
    public static final int BUILD_COOLDOWN = 1200;
    
    private IconButtonClickable[] selection;
    private IconSelection sellSelection;
    
    private BaseBuilding building = null;
    
    private boolean cooldown = false;
    private int buildCooldownStep = 0;
    
    public BaseBuildingSpot(Vector2 pos, Vector2 size, int team, PlayablePlayer owner, EngineControls engineControls, int buildingsBuildable) {
        super(pos, size, team, owner, engineControls);
        
        selection = new IconButtonClickable[]{
            new IconButtonClickable(Icon.FRANKIE, this.pos.cpy(), new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    tryBuiltPossibility(PossibleBaseBuildings.FRANKIE_FACTORY.name());
                    
                    disableShowingSelection();
                }

                @Override
                public boolean showTooltip() {
                    return true;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return new Tooltip(posx,
                            posy,
                            PossibleBaseBuildings.FRANKIE_FACTORY.getCost().coinsAmount,
                            PossibleBaseBuildings.FRANKIE_FACTORY.getCost().bonusAmount,
                            "Frankie Factory",
                            new String[] {"Spawns frankie every 10 seconds.", "Increases coin income by 20 per minute."});
                }

                @Override
                public Player getTooltipPlayer() {
                    return BaseBuildingSpot.this.owner;
                }
            }),
            
            new IconButtonClickable(Icon.MUTATE_LAB, this.pos.cpy(), new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    tryBuiltPossibility(PossibleBaseBuildings.MUTATE_LAB.name());
                    
                    disableShowingSelection();
                }
                
                @Override
                public boolean showTooltip() {
                    return true;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return new Tooltip(posx,
                            posy,
                            PossibleBaseBuildings.MUTATE_LAB.getCost().coinsAmount,
                            PossibleBaseBuildings.MUTATE_LAB.getCost().bonusAmount,
                            "Mutation Lab",
                            new String[] {"Allows you to mutate a unit"
                            , "for you that will spawn every 15 seconds."
                            , "Increases coin income by 40 per minute."});
                }

                @Override
                public Player getTooltipPlayer() {
                    return BaseBuildingSpot.this.owner;
                }
            }),
            
            new IconButtonClickable(Icon.CANCEL, this.pos.cpy(), new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    disableShowingSelection();
                }
                
                @Override
                public boolean showTooltip() {
                    return false;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return null;
                }

                @Override
                public Player getTooltipPlayer() {
                    return BaseBuildingSpot.this.owner;
                }
            })
        };
        
        if(buildingsBuildable > 0) {
            if(buildingsBuildable > selection.length - 1) {
                buildingsBuildable = selection.length - 1;
            }
            
            IconButtonClickable[] tempSelection = new IconButtonClickable[buildingsBuildable + 1];
            for(int i = 0; i < buildingsBuildable; i++) {
                tempSelection[i] = selection[i];
            }
            tempSelection[buildingsBuildable] = selection[selection.length - 1];
            selection = tempSelection;
        }
        
        this.initSelection(selection);
        
        sellSelection = new IconSelection(pos, new IconButtonClickable[] {
            new IconButtonClickable(Icon.DESTROY, pos, new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    removeBuilding();
                    hideSelection();
                }

                @Override
                public boolean showTooltip() {
                    return true;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return new Tooltip(posx, posy, 0, 0, "Remove building", new String[] {"Removes building on this building spot.",
                        "WARNING! You'll get no refund for it."});
                }

                @Override
                public Player getTooltipPlayer() {
                    return BaseBuildingSpot.this.owner;
                }
            }),
            new IconButtonClickable(Icon.CANCEL, pos, new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    hideSelection();
                }

                @Override
                public boolean showTooltip() {
                    return false;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return null;
                }

                @Override
                public Player getTooltipPlayer() {
                    return null;
                }
            })
        }, engineControls);
    }
    
    public void removeBuilding() {
        if(building != null) {
            if(!building.remove()) {
                building.forceRemove();
            }
        }
        building = null;
        cooldown = true;
        sellSelection.hide();
        
        engineControls.addParticle(new SmallExplosion(pos, size.x));
    }
    
    protected void hideSelection() {
        sellSelection.hide();
    }
    
    @Override
    public boolean canBeBuiltOn() {
        return (building == null && !cooldown);
    }
    
    public BaseBuilding getBuilding() {
        return building;
    }

    @Override
    public boolean hitsTexture(int posx, int posy) {
        if(posx >= pos.x - size.x/2 && posx <= pos.x + size.x/2 && posy >= pos.y - size.y/2 && posy <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void buildingSpotTick() {
        if(building != null) {
            if(building.remove()) {
                building = null;
                
                cooldown = true;
            }
        }
        
        if(cooldown) {
            buildCooldownStep++;
            if(buildCooldownStep >= BUILD_COOLDOWN) {
                buildCooldownStep = 0;
                cooldown = false;
            }
        }
    }

    @Override
    public void render() {
        if(building == null) {
            if(cooldown) {
                DrawableData.spriteBatch.setColor(0.8f, 0.2f, 0.2f, 1.0f);
            }
            
            RenderUtils.blit(DrawableData.spriteBatch,
                        BASE_BUILDING_SPOT_TEXTURE,
                        new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                        new RenderUtils.ScreenInfo(0, 0, TEXTURE_SIZE_X, TEXTURE_SIZE_Y));
            
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
    }

    @Override
    public boolean remove() {
        return false;
    }

    @Override
    public void onRemove() {
    }

    @Override
    public float getPriority() {
        return pos.y;
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }

    @Override
    public boolean spotHandle(MouseEvent me) {
        if(building != null) {
            if(building.showSellMenu() && me.mouseDown() && !building.isDead() && sellSelection.isHidden()) {
                if(building.hitsBuilding(me.getGameUnprojectedX(), me.getGameUnprojectedY())) {
                    owner.setIconSelectionAndShow(sellSelection);
                    return true;
                }
            } else {
                return building.handle(me);
            }
        }
        return false;
    }
    
    @Override
    public PossibilityCarrier[] getBuildingPossibilities() {
        List<PossibilityCarrier> outToArray = new ArrayList<PossibilityCarrier>();
        
        PossibleBaseBuildings[] possibleTowers = PossibleBaseBuildings.values();
        for(PossibleBaseBuildings possibleBaseBuilding : possibleTowers) {
            outToArray.add(possibleBaseBuilding.getPossibilityCarrier());
        }
        
        PossibilityCarrier[] out = new PossibilityCarrier[outToArray.size()];
        outToArray.toArray(out);
        return out;
    }

    @Override
    public void tryBuiltPossibility(String s) {
        if(canBeBuiltOn()) {
            final PossibleBaseBuildings pickedTower = PossibleBaseBuildings.valueOf(s);
            if (owner.getResourceManager().hasCurrencyFor(pickedTower.cost)) {
                owner.getResourceManager().removeCurrency(pickedTower.cost);
                
                engineControls.addParticle(new PurplePowderExplosion(pos, size.x));

                if(owner.isLocal()) {
                    SoundData.playRandomSound("work_1", "work_2");
                }
                
                switch (pickedTower) {
                    case FRANKIE_FACTORY:
                        building = new FrankieFactoryBuilding(pos,
                                FrankieFactoryBuilding.getDefaultSize(),
                                team,
                                (PlayablePlayer)owner,
                                engineControls);

                        engineControls.addEntity(building);
                        break;
                    case MUTATE_LAB:
                        building = new MutateLabBuilding(pos,
                                MutateLabBuilding.getDefaultSize(),
                                team,
                                (PlayablePlayer)owner,
                                engineControls,
                                BaseBuildingSpot.this);

                        engineControls.addEntity(building);
                        break;
                    default:
                        try {
                            throw new Exception("Missing handler for building " + pickedTower);
                        } catch (Exception ex) {
                            Logger.getLogger(TowerBuildingSpot.class.getName()).log(Level.SEVERE, null, ex);
                        }
                }
            } else {
                owner.addErrorMessage("Insufficient funds for this building.");
            }
        }
    }
}
