package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.particle.LightningParticle;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.List;

public class TeslaCoilTower extends TowerBuilding {
    public static Texture TESLA_COIL_TEXTURE = DrawableData.textures.get("tesla_coil");
    
    protected static enum State {
        STANDING,
        CHARGING,
        CHARGED;
    }
    
    protected static final int TEX_SIZE_X = 256;
    protected static final int TEX_SIZE_Y = 512;
    
    protected static final int TOWER_MAX_HEALTH = 300;
    protected static final int TOWER_ATTACK_CD = 8;
    protected static final int TOWER_ATTACK_DAMAGE = 5;
    protected static final float TOWER_ATTACK_RANGE = 305.0f;
    
    protected static final int[] CHARGING_ANIMATION_STEPS = {1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,6,6,6,6};
    protected static final int[] CHARGED_ANIMATION_STEPS = {6,6,6,6,7,7,7,7};
    
    protected static final int SOUND_COOLDOWN = 30;
    
    protected int health = TOWER_MAX_HEALTH;
    protected State state = State.STANDING;
    protected int currentAnimationStep = 0;
    
    protected Engine.EngineControls engineControls;
    
    protected boolean soundCd = false;
    protected int soundStep = 0;
    
    public TeslaCoilTower(Vector2 pos, Vector2 size, int team, Player owner, Engine.EngineControls engineControls) {
        super(pos, size, team, owner);
        this.engineControls = engineControls;
    }
    
    @Override
    public void towerTick() {
        List<AttackableEntity> inRangeTargets = engineControls.getEntityManager().getAttackableEntities(pos, TOWER_ATTACK_RANGE);
        if(state == State.STANDING) {
            for(final AttackableEntity e : inRangeTargets) {
                if(!e.isDead() && e != this && e.getTeam() != this.getTeam()) {
                    state = State.CHARGING;
                    currentAnimationStep = 0;
                    break;
                }
            }
        } else
        if(state == State.CHARGING) {
            currentAnimationStep++;
            if(currentAnimationStep >= CHARGING_ANIMATION_STEPS.length) {
                currentAnimationStep = 0;
                state = State.CHARGED;
            }
        } else
        if(state == State.CHARGED) {
            for(final AttackableEntity e : inRangeTargets) {
                if(!e.isDead() && e != this && e.getTeam() != this.getTeam()) {
                    tryAttack(e);
                    break;
                }
            }
            currentAnimationStep++;
            if(currentAnimationStep >= 360) {
                state = State.STANDING;
            }
        }
        
        if(soundCd) {
            soundStep++;
            if(soundStep > SOUND_COOLDOWN) {
                soundCd = false;
                soundStep = 0;
            }
        }
    }
    
    @Override
    public void render() {
        if(!isDead()) {
            switch(state) {
                case STANDING:
                    RenderUtils.blit(DrawableData.spriteBatch,
                            TESLA_COIL_TEXTURE,
                            new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2 - size.y, size.x, size.y + size.y),
                            new RenderUtils.ScreenInfo(0, 0, TEX_SIZE_X, TEX_SIZE_Y));
                    break;
                case CHARGING:
                    RenderUtils.blit(DrawableData.spriteBatch,
                            TESLA_COIL_TEXTURE,
                            new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2 - size.y, size.x, size.y + size.y),
                            new RenderUtils.ScreenInfo(TEX_SIZE_X * CHARGING_ANIMATION_STEPS[currentAnimationStep], 0, TEX_SIZE_X, TEX_SIZE_Y));
                    break;
                case CHARGED:
                    RenderUtils.blit(DrawableData.spriteBatch,
                            TESLA_COIL_TEXTURE,
                            new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2 - size.y, size.x, size.y + size.y),
                            new RenderUtils.ScreenInfo(TEX_SIZE_X * CHARGED_ANIMATION_STEPS[currentAnimationStep % CHARGED_ANIMATION_STEPS.length], 0, TEX_SIZE_X, TEX_SIZE_Y));
                    break;
            }
        }
    }
    
    @Override
    public boolean attack(AttackableEntity e) {
        if(!isDead()) {
            float distance = e.getPos().dst(pos);
            if(distance <= TOWER_ATTACK_RANGE) {
                float posOffsetX = 0.0f;//(20 + RandomGenerator.getRandom((int)size.x - 40)) - size.x/2;
                float posOffsetY = -(size.y + size.y/2.5f);
                
                Vector2 angleCalculator = e.getPos();
                angleCalculator.sub(pos).sub(posOffsetX, posOffsetY);
                float particleAngle = angleCalculator.angle();
                Vector2 particlePos = angleCalculator.cpy().mul(0.5f).add(pos).add(posOffsetX, posOffsetY);
                Vector2 particleSize = new Vector2(angleCalculator.len(),
                        angleCalculator.len() * (LightningParticle.TEX_SINGLE_HEIGHT)/LightningParticle.TEX_SINGLE_WIDTH);
                engineControls.addParticle(new LightningParticle(particlePos, particleSize, particleAngle));
                
                if(!soundCd) {
                    SoundData.playSound("electric_zap");
                    soundCd = true;
                }
                
                e.getHit(TOWER_ATTACK_DAMAGE, this);
                currentAnimationStep = 0;
                return true;
            }
        }
        
        return false;
    }

    @Override
    public int getAttackCooldown() {
        return TOWER_ATTACK_CD;
    }

    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            this.health -= damage;
        }
    }

    @Override
    public boolean isDead() {
        return (health <= 0);
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
    }
    
    @Override
    public boolean hitsTexture(int posx, int posy) {
        if(posx >= pos.x - size.x/2 && posx <= pos.x + size.x/2 && posy >= pos.y - size.y/2 && posy <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void forceRemove() {
        health = 0;
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(64.0f, 64.0f);
    }
}
