package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.particle.PurplePowderExplosion;
import com.pancik.libgdx.zumpatalentov.engine.particle.SmallExplosion;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager.CostWrapper;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconButtonClickable;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconSelection;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class TowerBuildingSpot extends BuildingSpotEntity {
    public static enum PossibleTowers {
        TESLA_COIL (new CostWrapper(35, 0)),
        TOFU_TOWER (new CostWrapper(100, 30));
        
        private CostWrapper cost;

        private PossibleTowers(CostWrapper cost) {
            this.cost = cost;
        }

        public CostWrapper getCost() {
            return cost;
        }

        public PossibilityCarrier getPossibilityCarrier() {
            PossibilityCarrier out = new PossibilityCarrier();
            out.name = name();
            out.cost = cost;
            return out;
        }
    }
    
    public static Texture TOWER_BUILDING_SPOT_TEXTURE = DrawableData.textures.get("tower_building_spot");
    
    public static final int TEXTURE_SIZE_X = 64;
    public static final int TEXTURE_SIZE_Y = 64;
    public static final float DEFAULT_SIZE_X = 64.0f;
    public static final float DEFAULT_SIZE_Y = 64.0f;
    public static final float OFFSET_Y = 0.0f;
    public static final float OFFSET_MULTIPLIER_Y = 0.0f;
    
    public static final int BUILD_COOLDOWN = 1200;
    
    private IconButtonClickable[] buildSelection;
    private IconSelection sellSelection;
    
    private TowerBuilding tower = null;
    
    private boolean cooldown = false;
    private int buildCooldownStep = 0;
    
    public TowerBuildingSpot(Vector2 pos, Vector2 size, int team, Player owner, Engine.EngineControls engineControls, int towersBuildable) {
        super(pos, size, team, owner, engineControls);
        
        buildSelection = new IconButtonClickable[]{
            new IconButtonClickable(Icon.TESLA_COIL, this.pos.cpy(), new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    tryBuiltPossibility(PossibleTowers.TESLA_COIL.name());
                    
                    disableShowingSelection();
                }
                
                @Override
                public boolean showTooltip() {
                    return true;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return new Tooltip(posx,
                            posy,
                            PossibleTowers.TESLA_COIL.getCost().coinsAmount,
                            PossibleTowers.TESLA_COIL.getCost().bonusAmount,
                            "Tesla coil",
                            new String[] {"Attacks random enemy unit in range.",
                            "With every shock it inflicts to target terrible pain."});
                }

                @Override
                public Player getTooltipPlayer() {
                    return TowerBuildingSpot.this.owner;
                }
            }),
            
            new IconButtonClickable(Icon.TOFU_TOWER, this.pos.cpy(), new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    tryBuiltPossibility(PossibleTowers.TOFU_TOWER.name());
                    
                    disableShowingSelection();
                }
                
                
                @Override
                public boolean showTooltip() {
                    return true;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return new Tooltip(posx,
                            posy,
                            PossibleTowers.TOFU_TOWER.getCost().coinsAmount,
                            PossibleTowers.TOFU_TOWER.getCost().bonusAmount,
                            "Tofu Tower",
                            new String[] {"Spawns tofu every 10 seconds.", "You'll need this. Trust me."});
                }

                @Override
                public Player getTooltipPlayer() {
                    return TowerBuildingSpot.this.owner;
                }
            }),
            
            new IconButtonClickable(Icon.CANCEL, this.pos.cpy(), new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    disableShowingSelection();
                }
                
                
                @Override
                public boolean showTooltip() {
                    return false;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return null;
                }

                @Override
                public Player getTooltipPlayer() {
                    return TowerBuildingSpot.this.owner;
                }
            })
        };
        
        if(towersBuildable > 0) {
            if(towersBuildable > buildSelection.length - 1) {
                towersBuildable = buildSelection.length - 1;
            }
            IconButtonClickable[] tempSelection = new IconButtonClickable[towersBuildable + 1];
            for(int i = 0; i < towersBuildable; i++) {
                tempSelection[i] = buildSelection[i];
            }
            tempSelection[towersBuildable] = buildSelection[buildSelection.length - 1];
            buildSelection = tempSelection;
        }
        
        this.initSelection(buildSelection);
        
        sellSelection = new IconSelection(pos, new IconButtonClickable[] {
            new IconButtonClickable(Icon.DESTROY, pos, new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    removeTower();
                    hideSelection();
                }

                @Override
                public boolean showTooltip() {
                    return true;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return new Tooltip(posx, posy, 0, 0, "Remove tower", new String[] {"Removes tower on this tower building spot.",
                        "WARNING! You'll get no refund for it."});
                }

                @Override
                public Player getTooltipPlayer() {
                    return TowerBuildingSpot.this.owner;
                }
            }),
            new IconButtonClickable(Icon.CANCEL, pos, new IconButtonClickable.CallbackHandler() {
                @Override
                public void onClicked() {
                    hideSelection();
                }

                @Override
                public boolean showTooltip() {
                    return false;
                }

                @Override
                public Tooltip getTooltip(float posx, float posy) {
                    return null;
                }

                @Override
                public Player getTooltipPlayer() {
                    return null;
                }
            })
        }, engineControls);
    };

    @Override
    public boolean hitsTexture(int posx, int posy) {
        if(posx >= pos.x - size.x/2 && posx <= pos.x + size.x/2 && posy >= pos.y - size.y/2 && posy <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void buildingSpotTick() {
        if(tower != null) {
            if(tower.remove()) {
                removeTower();
            }
        }
        
        if(cooldown) {
            buildCooldownStep++;
            if(buildCooldownStep >= BUILD_COOLDOWN) {
                buildCooldownStep = 0;
                cooldown = false;
            }
        }
    }

    @Override
    public void render() {
        if(tower == null) {
            if(cooldown) {
                DrawableData.spriteBatch.setColor(0.8f, 0.2f, 0.2f, 1.0f);
            }
            RenderUtils.blit(DrawableData.spriteBatch,
                    TOWER_BUILDING_SPOT_TEXTURE,
                    new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2 - (size.y * OFFSET_Y), size.x, size.y + (size.y * (OFFSET_MULTIPLIER_Y))),
                    new RenderUtils.ScreenInfo(0, 0, TEXTURE_SIZE_X, TEXTURE_SIZE_Y));
            
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
    }
    
    protected void removeTower() {
        if(tower != null) {
            if(!tower.remove()) {
                tower.forceRemove();
            }
        }
        tower = null;
        cooldown = true;
        sellSelection.hide();
        
        engineControls.addParticle(new SmallExplosion(pos, size.x));
    }
    
    protected void hideSelection() {
        sellSelection.hide();
    }
    
    @Override
    public boolean remove() {
        return false;
    }

    @Override
    public void onRemove() {
    }
    
    @Override
    public float getPriority() {
        return pos.y;
    }

    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }

    @Override
    public boolean canBeBuiltOn() {
        return (tower == null && !cooldown);
    }

    @Override
    public boolean spotHandle(MouseEvent me) {
        if(tower != null && me.mouseDown()) {
            if(!tower.isDead() && sellSelection.isHidden()) {
                if(tower.hitsTexture(me.getGameUnprojectedX(), me.getGameUnprojectedY())) {
                    owner.setIconSelectionAndShow(sellSelection);
                    return true;
                }
            }
        }
        
        return false;
    }
    
    @Override
    public PossibilityCarrier[] getBuildingPossibilities() {
        List<PossibilityCarrier> outToArray = new ArrayList<PossibilityCarrier>();
        
        PossibleTowers[] possibleTowers = PossibleTowers.values();
        for(PossibleTowers possibleTower : possibleTowers) {
            outToArray.add(possibleTower.getPossibilityCarrier());
        }
        
        PossibilityCarrier[] out = new PossibilityCarrier[outToArray.size()];
        outToArray.toArray(out);
        return out;
    }

    @Override
    public void tryBuiltPossibility(String s) {
        if(canBeBuiltOn()) {
            final PossibleTowers pickedTower = PossibleTowers.valueOf(s);
            if (owner.getResourceManager().hasCurrencyFor(pickedTower.cost)) {
                owner.getResourceManager().removeCurrency(pickedTower.cost);

                engineControls.addParticle(new PurplePowderExplosion(pos, size.x));
                
                if(owner.isLocal()) {
                    SoundData.playRandomSound("work_1", "work_2");
                }

                switch (pickedTower) {
                    case TESLA_COIL:
                        tower = new TeslaCoilTower(pos,
                                TeslaCoilTower.getDefaultSize(),
                                team,
                                owner,
                                engineControls);

                        engineControls.addEntity(tower);
                        break;
                    case TOFU_TOWER:
                        tower = new TofuTower(pos,
                                TofuTower.getDefaultSize(),
                                team,
                                owner);

                        engineControls.addEntity(tower);
                        break;
                    default:
                        try {
                            throw new Exception("Missing handler for tower " + pickedTower);
                        } catch (Exception ex) {
                            Logger.getLogger(TowerBuildingSpot.class.getName()).log(Level.SEVERE, null, ex);
                        }
                }
            } else {
                owner.addErrorMessage("Insufficient funds for this tower.");
            }
        }
    }
}
