package com.pancik.libgdx.zumpatalentov.engine.component;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.tile.Tile;
import com.pancik.libgdx.zumpatalentov.entity.Entity;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class EntityManager {
    protected static final float DAMPING_E_TO_E = 0.0125f;
    protected static final float DAMPING_E_TO_W = 0.10f;
    
    private final List<Entity> entities = new ArrayList<Entity>();
    private TileManager tileManager;

    public EntityManager(TileManager tileManager) {
        this.tileManager = tileManager;
    }
    
    public void tick() {
        for(int i = 0; i < entities.size(); i++) {
            entities.get(i).tick();
        }
        
        ArrayList<Entity> shouldBeRemoved = new ArrayList<Entity>();

        for(Entity e : entities) {
            if(e.remove()) {
                shouldBeRemoved.add(e);
            }
        }

        for(Entity e : shouldBeRemoved) {
            e.onRemove();
            entities.remove(e);
        }
        
        //after removal, push away remaining unit entities
        pushAwayUnitEntities();
        //System.out.println("Entities count: " + entities.size());
    }
    
    public void render() {
        Collections.sort(entities, new Comparator<Entity>() {
            @Override
            public int compare(Entity e1, Entity e2) {
                if(e1.getPriority() < e2.getPriority()) {
                    return -1;
                } else if (e1.getPriority() > e2.getPriority()) {
                    return 1;
                } else {
                    return 0;
                }
            }
        });

        for(Entity e : entities) {
            e.render();
        }
    }
    
    public void addEntity(Entity e) {
        entities.add(e);
    }
    
    public void removeEntity(Entity e) {
        entities.remove(e);
    }
    
    public Entity findById(Long id) {
        Entity out = null;
        
        for(Entity entity : entities) {
            if(entity.getId().equals(id)) {
                out = entity;
                break;
            }
        }
        
        return out;
    }
    
    public void pushAwayUnitEntities() {
        List<UnitEntity> unitEntities = new ArrayList<UnitEntity>();
        for(Entity e : entities) {
            if(e instanceof UnitEntity) {
                UnitEntity unitEntity = (UnitEntity)e;
                unitEntities.add(unitEntity);
            }
        }
        
        for(int j = 0; j < unitEntities.size(); j++) {
            for(int i = j + 1; i < unitEntities.size(); i++) {
                UnitEntity e1 = unitEntities.get(j);
                UnitEntity e2 = unitEntities.get(i);
                
                Vector2 pos1 = e1.getPos();
                Vector2 pos2 = e2.getPos();
                if(pos1.dst(pos2) <= e1.getDiameter()/2 + e2.getDiameter()/2) {
                    //push away 1
                    Vector2 move1 = pos2.cpy();
                    move1.sub(pos1);
                    //move1.nor();
                    move1.mul(DAMPING_E_TO_E);
                    
                    //push away 2
                    Vector2 move2 = pos1.cpy();
                    move2.sub(pos2);
                    //move2.nor();
                    move2.mul(DAMPING_E_TO_E);
                    
                    e2.addPos(move1);
                    e1.addPos(move2);
                }
            }
        }
        
        for(int i = 0; i < unitEntities.size(); i++) {
            Vector2 pos1 = unitEntities.get(i).getPos().add(0.0f, unitEntities.get(i).getSize().y/3);
            
            Tile[] surroundings = tileManager.getSurroundingTiles(pos1.x, pos1.y);
            for(Tile tile : surroundings) {
                if(tile != null) {
                    if(!tile.isWalkable()) {
                        Vector2 pos2 = new Vector2(tile.getPosx(), tile.getPosy());

                        if(pos1.dst(pos2) <= unitEntities.get(i).getDiameter()/2 + Tile.TILE_SIZE/2) {
                            Vector2 move2 = pos1.cpy();
                            move2.sub(pos2);
                            move2.mul(DAMPING_E_TO_W);

                            unitEntities.get(i).addPos(move2);
                        }
                    }
                }
            } 
        }
    }
    
    public List<AttackableEntity> getAttackableEntities(Vector2 from, float range) {
        List<AttackableEntity> out = new ArrayList<AttackableEntity>();
        
        for(Entity e : entities) {
            if(e instanceof AttackableEntity) {
                AttackableEntity attackableEntity = (AttackableEntity)e;
                
                if(attackableEntity.getPos().dst(from) <= range) {
                    out.add(attackableEntity);
                }
            }
        }
        
        return out;
    }
}
