package com.pancik.libgdx.zumpatalentov.entity.unit;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.Level;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.entity.UnitEntityAI;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import java.util.HashMap;
import java.util.Map.Entry;

public abstract class UnitEntity extends AttackableEntity {
    public static final float MELEE_ATTACK_RANGE = 40.0f;
    
    public static enum MoveType {
        MOVE_LEFT,
        MOVE_UP,
        MOVE_RIGHT,
        MOVE_DOWN;
    }
    
    public static class MoveResult {
        public MoveType move;
        public Vector2 result;
    }
    
    protected UnitEntityAI ai;
    protected Engine.EngineControls engineControls;
    
    protected HashMap<Class<? extends Buff>, Buff> buffs = new HashMap<Class<? extends Buff>, Buff>();
    
    protected boolean moved = false;
    protected boolean attacked = false;
    
    protected int attackCdStep = 0;

    public UnitEntity(Vector2 pos, Vector2 size, int team, Player owner, Engine.EngineControls engineControls) {
        super(pos, size, team, owner);
        this.engineControls = engineControls;
    }

    protected void setAi(UnitEntityAI ai) {
        this.ai = ai;
    }
    
    @Override
    public void tick() {
        for(Entry<Class<? extends Buff>, Buff> entry : buffs.entrySet()) {
            if(entry.getValue() != null) {
                entry.getValue().tick();
            }
        }
        
        for(Entry<Class<? extends Buff>, Buff> entry : buffs.entrySet()) {
            if(entry.getValue() != null) {
                if(entry.getValue().remove()) {
                    entry.setValue(null);
                }
            }
        }
        
        moved = false;
        
        if(attacked) {
            attackCdStep++;
            if(attackCdStep >= getAttackCooldown()) {
                attacked = false;
            }
        }
        
        if(ai != null) {
            ai.tick(this);
        }
        
        unitTick();
    }

    public void tryMoveUp() {
        if(!moved && !attacked) {
            moveUp();
            moved = true;
        }
    }
    
    public void tryMoveDown() {
        if(!moved && !attacked) {
            moveDown();
            moved = true;
        }
    }
    
    public void tryMoveRight() {
        if(!moved && !attacked) {
            moveRight();
            moved = true;
        }
    }
    
    public void tryMoveLeft() {
        if(!moved && !attacked) {
            moveLeft();
            moved = true;
        }
    }
    
    public boolean tryAttack(AttackableEntity e) {
        if(!attacked && (e.getTeam() != this.getTeam())) {
            if(attack(e)) {

                attacked = true;
                attackCdStep = 0;
                return true;
            }
        }
        
        return false;
    }

    @Override
    public void render() {
        for(Entry<Class<? extends Buff>, Buff> entry : buffs.entrySet()) {
            if(entry.getValue() != null) {
                if(!entry.getValue().remove()) {
                    entry.getValue().preRender();
                }
            }
        }
        
        unitRender();
        
        for(Entry<Class<? extends Buff>, Buff> entry : buffs.entrySet()) {
            if(entry.getValue() != null) {
                if(!entry.getValue().remove()) {
                    entry.getValue().postRender();
                }
            }
        }
    }
    
    protected void addBuff(Buff buff) {
        buffs.put(buff.getClass(), buff);
    }
    
    public float getBuffCoefficient(Buff.PercentageStat stat) {
        float out = 0.0f;
        
        out += owner.getUpgradeCoefficient(stat);
        
        for(Entry<Class<? extends Buff>, Buff> entry : buffs.entrySet()) {
            if(entry.getValue() != null) {
                if(!entry.getValue().remove()) {
                    out += entry.getValue().getCoefficient(stat);
                }
            }
        }
        
        return out;
    }

    public abstract float getDiameter();
    protected abstract void unitTick();
    protected abstract void unitRender();

    protected abstract int getAttackCooldown();
    public abstract float getAttackRange();
    public abstract float getAttackSightRange();
    protected abstract boolean attack(AttackableEntity e);
    
    protected abstract void moveUp();
    protected abstract void moveDown();
    protected abstract void moveRight();
    protected abstract void moveLeft();

    public abstract void setWaypointPath(Level.WaypointPath waypointPath);
    public abstract MoveResult[] getMoveResults();
    public abstract void apply(MoveType type);
}
