package com.pancik.libgdx.zumpatalentov.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.Color;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.MainApplicationListener;
import com.pancik.libgdx.zumpatalentov.util.KeyChangedEvent;
import com.pancik.libgdx.zumpatalentov.util.KeyTypedEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.LinkedList;

public class TextForm {
    public static interface CallbackHandler {
        public void onSubmit(String text);
    }
    
    private String placeholder;
    private LinkedList<Character> text;
    private ButtonHorizontalAlignment horizontal;
    private int deltax;
    private ButtonVerticalAlignment vertical;
    private int deltay;
    private CallbackHandler callback;

    private int cursorPosx;
    private int cursorStep = 0;
    private final int cursorStepCount = 30;
    private boolean cursorShow = true;

    private int deleteStep = 0;
    private final int deleteStepCount = 6;
    private boolean deletePressed = false;

    private boolean focus = false;

    private int maxTextSize;
    private boolean maxTextSizeOverstepped = false;

    public static final int formWidth = 600;
    public static final int formHeight = 105;

    private static final int formBegx = 0;
    private static final int formBegy = 0;
    private static final int formEndx = 800;
    private static final int formEndy = 200;

    private static final int cursorBegx = 800;
    private static final int cursorBegy = 0;
    private static final int cursorEndx = 15;
    private static final int cursorEndy = 192;

    public static final String possibleCharacters = "abcdefghijklmnoprstquvwxyzABCDEFGHIJKLMNOPRSTQUVWXYZ0123456789!@#$^&()_-;, ";

    public static int getDensityFormWidth() {
        return (int)(formWidth * DrawableData.getDensity());
    }

    public static int getDensityFormHeight() {
        return (int)(formHeight * DrawableData.getDensity());
    }

    public TextForm(String placeholder, ButtonHorizontalAlignment horizontal, int deltax, ButtonVerticalAlignment vertical, int deltay, int maxTextSize, CallbackHandler callback) {
        this.horizontal = horizontal;
        this.deltax = deltax;
        this.vertical = vertical;
        this.deltay = deltay;
        this.callback = callback;
        this.text = new LinkedList<Character>();
        this.placeholder = placeholder;
        this.maxTextSize = maxTextSize;

        recalculateCursorPos();
    }

    private void deleteLastChar() {
        if(focus && !text.isEmpty()) {
            text.removeLast();
            recalculateCursorPos();
        }
    }

    public void handle(KeyChangedEvent e) {
        if(e.isDown()) {
            switch(e.getKeyPressed()) {
                case Input.Keys.BACKSPACE:
                    deleteStep = 0;
                    deletePressed = true;
                    deleteLastChar();
                    break;
                case Input.Keys.ENTER:
                    if(focus && callback != null) {
                        callback.onSubmit(getText());
                    }
                    break;
            }
        } else {
            switch(e.getKeyPressed()) {
                case Input.Keys.BACKSPACE:
                    deletePressed = false;
                    break;
            }
        }
    }

    public void handle(KeyTypedEvent e) {
        if(focus) {
            if(possibleCharacters.indexOf(e.getKeyPressed()) != -1) {
                if(text.size() < maxTextSize) {
                    text.add(e.getKeyPressed());
                    recalculateCursorPos();
                } else {
                    maxTextSizeOverstepped = true;
                }
            }
        }
    }

    public void handle(MouseEvent e) {
        if(e.mouseDown()) {
            if(checkIfClicked(e.getUiUnprojectedX(), e.getUiUnprojectedY())) {
                focus = true;
                Gdx.input.setOnscreenKeyboardVisible(true);
                cursorStep = 0;
            } else {
                //lost focus
                focus = false;
            }
        }
    }

    public void tick() {
        if(deletePressed) {
            deleteStep++;
            if(deleteStep >= deleteStepCount) {
                deleteStep = 0;
                deleteLastChar();
            }
        }

        cursorStep++;
        if(cursorStep >= cursorStepCount) {
            if(cursorShow) {
                cursorShow = false;
            } else {
                cursorShow = true;
            }
            cursorStep = 0;
        }

        maxTextSizeOverstepped = false;
    }

    public void render() {
        RenderUtils.blit(DrawableData.spriteBatch,
                DrawableData.textures.get("form"),
                new RenderUtils.ScreenInfo(calculateX(), calculateY(), getDensityFormWidth(), getDensityFormHeight()),
                new RenderUtils.ScreenInfo(formBegx, formBegy, formEndx, formEndy));

        if(getText().length() != 0) {
            RenderUtils.blitText(DrawableData.spriteBatch,
                    getText(),
                    calculateX(),
                    calculateY() + getDensityFormHeight() - 18,
                    RenderUtils.TextAlligment.LEFT,
                    RenderUtils.TextAlligment.TOP);
        } else {
            if(!focus) {
                RenderUtils.blitText(DrawableData.spriteBatch,
                    placeholder,
                    calculateX(),
                    calculateY() + getDensityFormHeight() - 15,
                    Color.GRAY,
                    RenderUtils.TextAlligment.LEFT,
                    RenderUtils.TextAlligment.TOP);
            }
        }

        if(focus && cursorShow) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("form"),
                    new RenderUtils.ScreenInfo(calculateX() + cursorPosx,
                        calculateY() + (int)(56 * DrawableData.getDensity()),
                        (int)(4 * DrawableData.getDensity()),
                        (int)(38 * DrawableData.getDensity())),
                    new RenderUtils.ScreenInfo(cursorBegx, cursorBegy, cursorEndx, cursorEndy));
        }
    }

    private String getText() {
        StringBuilder out = new StringBuilder();
        for(Character c : text) {
            out.append(c);
        }
        return out.toString();
    }

    public void setText(String s) {
        text.clear();
        for(int i = 0; i < s.length(); i++) {
            text.add(s.charAt(i));
        }
        recalculateCursorPos();
    }
    
    public boolean isMaxTextSizeOverstepped() {
        return maxTextSizeOverstepped;
    }

    public boolean checkIfClicked(float x, float y) {
        if (calculateX() < x && calculateX() + getDensityFormWidth() > x && calculateY() < y && calculateY() + getDensityFormHeight() > y) {
            return true;
        } else {
            return false;
        }
    }

    private void recalculateCursorPos() {
        String composedText = getText();

        cursorPosx = (int)DrawableData.blackFont.getBounds(composedText).width + 3;
    }

    private int calculateX() {
        return RenderUtils.calculateX(horizontal, deltax, getDensityFormWidth());
    }

    private int calculateY() {
        return RenderUtils.calculateY(vertical, deltay, getDensityFormHeight());
    }
}
